package IVYZ

import (
	"context"
	"encoding/hex"
	"encoding/json"
	"tianyuan-api/apps/api/internal/common"
	"tianyuan-api/apps/api/internal/svc"
	"tianyuan-api/apps/api/internal/types"
	"tianyuan-api/apps/api/internal/validator"
	"tianyuan-api/apps/api/internal/westmodel"
	"tianyuan-api/pkg/crypto"
	"tianyuan-api/pkg/errs"

	"github.com/zeromicro/go-zero/core/logx"
)

type IVYZ5733Logic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewIVYZ5733Logic(ctx context.Context, svcCtx *svc.ServiceContext) *IVYZ5733Logic {
	return &IVYZ5733Logic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *IVYZ5733Logic) IVYZ5733(req *types.Request) (resp *types.Response, err *errs.AppError) {
	secretKey, ok := l.ctx.Value("secretKey").(string)
	if !ok {
		return &types.Response{}, errs.ErrSystem
	}

	transactionID, ok := l.ctx.Value("transactionID").(string)
	if !ok {
		return &types.Response{}, errs.ErrSystem
	}

	// 1、解密
	key, decodeErr := hex.DecodeString(secretKey)
	if decodeErr != nil {
		return nil, errs.ErrSystem
	}
	decryptData, aesDecryptErr := crypto.AesDecrypt(req.Data, key)
	if aesDecryptErr != nil || len(decryptData) == 0 {
		return nil, errs.ErrParamDecryption
	}

	// 2、校验
	var data validator.IVYZ5733Request
	if validatorErr := validator.ValidateAndParse(decryptData, &data); validatorErr != nil {
		return nil, errs.ErrParamValidation
	}

	// 3、西部加密
	westConfig := l.svcCtx.Config.WestConfig
	encryptedFields, encryptStructFieldsErr := common.EncryptStructFields(data, westConfig.Key)
	if encryptStructFieldsErr != nil {
		logx.Errorf("西部加密错误：%v", err)
		return nil, errs.ErrSystem
	}

	logx.Infof("交易号：%s", transactionID)
	apiRequest := common.MapStructToAPIRequest(encryptedFields, westmodel.IVYZ5733FieldMapping, "data")
	westResp, callAPIErr := l.svcCtx.WestDexService.CallAPI("G09GX01", apiRequest)
	if callAPIErr != nil {
		return nil, errs.ErrSystem
	}
	// 5、响应解析
	var respData westmodel.G09GX01Response
	unmarshalErr := json.Unmarshal(westResp, &respData)
	if unmarshalErr != nil {
		return nil, errs.ErrSystem
	}

	if respData.Code == 400 || respData.Code == 500 || respData.Code == 999 {
		logx.Errorf("西部响应错误%v", respData)
		return nil, errs.ErrSystem
	}
	if respData.Code == 201 || respData.Code == 202 || respData.Code == 203 {
		l.ctx = context.WithValue(l.ctx, "Charges", true)
	}
	encryptData, aesEncrypt := crypto.AesEncrypt(westResp, key)
	if aesEncrypt != nil {
		return nil, errs.ErrSystem
	}
	return &types.Response{
		Data: encryptData,
	}, nil
}
